<?php

namespace Drupal\standwithukraine\Service;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Elphin\IcoFileLoader\IcoFileService;

/**
 * Class StandWithUkraineFavicon.
 *
 * @package Drupal\standwithukraine\Service
 */
class StandWithUkraineFavicon implements StandWithUkraineFaviconInterface {

  /**
   * The flag colors.
   */
  const COLORS = [
    TRUE => [0x00, 0x5B, 0xBB],
    FALSE => [0xFF, 0xD5, 0x00],
  ];

  /**
   * The cache.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cache;

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The file URL generator.
   *
   * @var \Drupal\Core\File\FileUrlGeneratorInterface|null
   */
  protected $fileUrlGenerator;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    CacheBackendInterface $cache,
    ConfigFactoryInterface $config_factory,
    FileUrlGeneratorInterface $file_url_generator = NULL
  ) {
    $this->cache = $cache;
    $this->configFactory = $config_factory;
    $this->fileUrlGenerator = $file_url_generator;
  }

  /**
   * {@inheritdoc}
   */
  public function build($theme) {
    $cache = $this->cache->get('standwithukraine');

    if ($cache !== FALSE) {
      $items = $cache->data;

      if (isset($items[$theme])) {
        return $items;
      }
    }
    else {
      $items = [];
    }

    if (($filename = theme_get_setting('favicon.url', $theme)) === NULL) {
      return $items;
    }

    $loader = new IcoFileService();

    try {
      $icon = $loader->from(DRUPAL_ROOT . '/' . $filename);
    }
    catch (\Exception $exception) {
      return $items;
    }

    $colors = [];

    foreach (self::COLORS as $delta => $color) {
      $colors[$delta] = '';

      foreach (array_reverse($color) as $part) {
        $colors[$delta] .= chr($part);
      }
    }

    $config = $this->configFactory->get('standwithukraine.settings');
    $ratio = $config->get('favicon.ratios.' . $theme) ?? self::UNDEFINED_RATIO;

    /** @var \Elphin\IcoFileLoader\IconImage $image */
    foreach ($icon as $image) {
      if (!$image->isBmp() || $image->bitCount !== 32) {
        continue;
      }

      $middle = $image->height / 100 * $ratio;
      $delta = 0;
      $bitmap = '';

      for ($vertical = $image->height; $vertical > 0; $vertical--) {
        for ($horizontal = 0; $horizontal < $image->width; $horizontal++) {
          $bitmap .= $colors[$vertical < $middle];
          $delta += 3;
          $bitmap .= $image->bmpData[$delta++];
        }
      }

      $image->setBitmapData($bitmap);

      $filename = sprintf(
        'public://standwithukraine-favicon-%s-%s.png',
        $theme,
        $dimensions = $image->width . 'x' . $image->height
      );

      imagepng($loader->renderImage($image), $filename);

      $items[$theme][$dimensions] = $this->fileUrlGenerator !== NULL
        ? $this->fileUrlGenerator->generateString($filename)
        : file_create_url($filename);
    }

    $this->cache->set('standwithukraine', $items);

    return $items;
  }

}
